﻿-- Wowzzle
-- by Fluffies EU - Well of Eternity
local addonName, addonTable = ...

addonTable.version = 1003
local C = addonTable.C -- communication commands
local L = addonTable.L -- localization

local min = math.min
local abs = math.abs
local rnd = math.random
local low = string.lower
local upp = string.upper

local framewidth  = 300
local frameheight = 300
local lines = 5
local rows  = 5
local buttonwidth  = framewidth / rows
local buttonheight = frameheight / lines

local game_timelimit = 120
local time_word_fast  = 0.5
local time_word_slow  = 2
local time_word_wrong = 2

local scorecolor = "|cffaa5577"

local buttons    = {}
local path       = {}
local pathS      = ""
local pathBonus  = 0
local wordsfound = {}
local wordsavailable = {}
local totalscore = 0

-- word state
local NONE = 0
local COMPLETING = 1
local COMPLETE = 2
local BOTH = 3 -- complete but also completing a longer word

-- timers
local TIMER_GAME   = 1
local TIMER_COMM   = 2
local TIMER_GLOBAL = 3

-- game state
local gametype = "singleplayer"

-- indexed letters
local Iletters = {"A","B","C","D","E","F","G","H","J","K","I","L","M",
                  "N","O","P","Q","R","S","T","U","V","W","X","Y","Z"}
-- points per letter
local letters = {
["A"] = 1,["B"] = 4,["C"] = 1,["D"] = 4,["E"] = 1,["F"] = 4,
["G"] = 4,["H"] = 8,["J"] = 8,["K"] = 8,["I"] = 1,["L"] = 2,
["M"] = 2,["N"] = 2,["O"] = 1,["P"] = 3,["Q"] = 10,["R"] = 1,
["S"] = 1,["T"] = 1,["U"] = 4,["V"] = 4,["W"] = 6,["X"] = 8,
["Y"] = 4,["Z"] = 8}

-- single button
local function SetButtonColor(r,g,b,btn)
 btn.background:SetVertexColor(r,g,b, 0.8)
end

local function SetButtonsColor(r,g,b,selectedonly, glow, deselect)
 table.foreach(buttons,
  function(k,v)
   if (selectedonly and v.button.selected) or not selectedonly then
    SetButtonColor(r,g,b,v.button)
   if glow then
    ActionButton_ShowOverlayGlow(v.button);
   else
    ActionButton_HideOverlayGlow(v.button);
   end
   end
   if deselect then
    v.button.selected = false
   end
  end)
end

local function EnableButtons()
 table.foreach(buttons, function(_,v)
  v:Show()
 end)
end

local function DisableButtons()
 table.foreach(buttons, function(_,v)
  v:Hide()
 end)
end

addonTable.RefreshSideFrames = function()
 if Wowzzle_Mainframe then
  Wowzzle_Mainframe.leftframe.Refresh()
  Wowzzle_Mainframe.rightframe.Refresh()
 end
end

local function ResetPath()
 wipe(path)
 pathS = ""
 SetButtonsColor(1,1,1,false,false,true)
 addonTable:Timer_Reset(TIMER_GAME)
 if Wowzzle_Mainframe then
  Wowzzle_Mainframe.currentword:SetText("")
 end
end

local function ResetGame()
 ResetPath()
 totalscore = 0
 wipe(wordsfound)
 wipe(wordsavailable)
 addonTable.RefreshSideFrames()
 DisableButtons()
 addonTable.Timer_Reset(TIMER_GLOBAL)
 addonTable.opponent.score = 0
end

--[[
length bonus
5 letters: 3 points
6 letters: 6 points
7 letters: 9 points
8 letters: 12 points.
]]
local function AssignScore()
 local word = pathS
 local c
 local score = 0
 local bonus = 0
 for c in string.gmatch(word,"(%a)") do
  score = score + letters[upp(c)]
 end
 addonTable.CreateFloatingText(Wowzzle_Mainframe.leftframe, score, 4)
 local l = strlen(word)
 if l > 7 then
  bonus = 12
 elseif l > 6 then
  bonus = 9
 elseif l > 5 then
  bonus = 6
 elseif l > 4 then
  bonus = 3
 end
 ResetPath()
 score = score + bonus
 totalscore = totalscore + score
 addonTable.RefreshSideFrames()
 table.insert(wordsfound, low(word))
 if gametype == "multiplayer" then
  addonTable.SendToOpponent(C["newscore"], score)
 end
 return score
end

local function CompareOutput(suff)
 local w = low(pathS..suff)

 local function CheckTable(t)
  local result = NONE
  if addonTable.wowzzle_database[t] then
   local word
   for i,word in pairs(addonTable.wowzzle_database[t]) do
   --for i,word in pairs(wowzzle_database) do -- tmp
    if not tContains(wordsfound, word) and strfind(word,"^"..w) then
     if word == w then -- got it!
      local s
      for j,s in pairs(addonTable.wowzzle_database[t]) do
       if j ~= i and not tContains(wordsfound, s) and strfind(s,"^"..w) then
        return BOTH
       end
      end
      return COMPLETE
     else -- word not complete
      result = (result == COMPLETE) and BOTH or COMPLETING
     end
    end
   end
  end
  return result
 end
 
 local r = CheckTable(upp(w:sub(1,1)))
 --[[if r ~= COMPLETE then
  r = CheckTable("ZONES")
 end
 if r ~= COMPLETE then
  r = CheckTable("SPELLS")
 end]]--

 return r
end

-- dispose frames
addonTable.Wowzzle_Dispose = function()
  ResetPath()
  table.foreach(buttons, function(k,v)
   v.button.text = nil
   v.button.background = nil
   v.button.selected = nil
   v.button.text_points = nil
   v.button = nil
   v = nil
  end)
  wipe(buttons)
  wipe(wordsfound)
  addonTable.Dispose_FT()
  addonTable.ResetTimers()

  if Wowzzle_Mainframe then
   Wowzzle_Mainframe.buttonsframe = nil
   Wowzzle_Mainframe.closebtn = nil
   Wowzzle_Mainframe:Hide()
   Wowzzle_Mainframe = nil
  end

  addonTable.DisposeMenu()
  collectgarbage()
end
--

-- buttonsframe constructor
local function AddButtontoPath(btn, x, y)
 local letter = btn.text:GetText()
 if letter then
  btn.selected = true
  btn.background:SetVertexColor(0, 0.5, 0.9, 1)
  local lastpos = path[table.getn(path)]
  table.insert(path, {x,y})
  pathS = pathS..letter
  Wowzzle_Mainframe.currentword:SetText(pathS)
  end
end

-- Button Clicks
local function ProcessRightClick()
 if not addonTable.Timer_Running(TIMER_GAME) then
  ResetPath()
 end
end

local function ProcessButtonClick(btn, x, y)
 if GetMouseButtonClicked() == "RightButton" then
  ProcessRightClick()
  return
 end

 local pcount = table.getn(path)

 local allowed = ((not btn.selected) and
                 ((pcount == 0)     -- no letters selected..
              or (    (abs(path[pcount][1] - x) <= 1 and abs(path[pcount][2] - y) <= 1)
                  and (abs(path[pcount][1] - x) + abs(path[pcount][2] - y) > 0))) ) -- ..in proximity of the last one

 if allowed then
  local r = CompareOutput(btn.text:GetText())
   -- result
   if r == COMPLETE or r == BOTH then
    AddButtontoPath(btn, x, y)
    SetButtonsColor(0,1,0, true, true)
    PlaySoundFile("Sound\\interface\\MagicClick.wav")
    return addonTable.Timer_Start(TIMER_GAME, r == COMPLETE and time_word_fast or time_word_slow, AssignScore, "Sound\\Spells\\PVPFlagCaptured.wav")
   elseif r == COMPLETING then
    AddButtontoPath(btn, x, y)
    SetButtonsColor(0, 0.2, 0.4, true)
    SetButtonColor(0, 0.6, 1, btn)
    addonTable.Timer_Reset(TIMER_GAME)
    PlaySoundFile("Sound\\Spells\\SimonGame_Visual_GameTick.wav")
   elseif r == NONE and not addonTable.Timer_Running(TIMER_GAME) then -- no matches
    AddButtontoPath(btn, x, y)
    SetButtonsColor(1,0,0, true, true)
    PlaySoundFile("Sound\\Spells\\CounterSpell.wav")
    addonTable.Timer_Start(TIMER_GAME, time_word_wrong, ResetPath)
   end
 else
  -- wrong selection
 end
end
--

local function GenerateGrid()
 table.foreach(buttons, function(_,v)
  v.button.text:SetText("")
 end)
 addonTable.SetGrid(addonTable.GenerateGrid())
end

local function GenerateButtonsframe(parent)

 local function createbutton(p, x, y)
 -- border highlight frame
  local f = CreateFrame("FRAME")
  f:SetParent(p)
  f:SetHeight(buttonwidth)
  f:SetWidth(buttonheight)
  f:SetPoint("TOPLEFT", p, "TOPLEFT", x * buttonwidth, -(y * buttonheight))
  f.tx = f:CreateTexture()
  f.tx:SetTexture(1,1,1,0)
  f.tx:SetAllPoints()
  f:SetScript("OnMouseDown", function() 
  if GetMouseButtonClicked() == "RightButton" then
   ProcessRightClick()
  end
 end)

 -- the button itself
  f.button = CreateFrame("BUTTON", "Wowzzle_button"..x..y, f, "OptionsListButtonTemplate");
  f.button:SetHeight(buttonwidth - 4)
  f.button:SetWidth(buttonheight - 4)  
  f.button:SetPoint("CENTER")
  
  -- shadow
  f.button.shadow = f:CreateTexture()
  f.button.shadow:SetTexture("interface\\addons\\wowzzle\\shadow.tga")
  f.button.shadow:SetBlendMode("BLEND")
  f.button.shadow:SetPoint("CENTER", f.button, "CENTER", -3, -3)
  f.button.shadow:SetHeight(buttonwidth + 5)
  f.button.shadow:SetWidth(buttonheight + 5)
  -- background
  f.button.background = f.button:CreateTexture()
  f.button.background:SetTexture("interface\\addons\\wowzzle\\btn.tga")
  f.button.background:SetBlendMode("BLEND")
  SetButtonColor(1,1,1,f.button)
  f.button.background:SetAllPoints()

  -- text, used to store and show the button's letter
  f.button.text = f.button:CreateFontString(nil, "OVERLAY")
  f.button.text:SetAllPoints()
  f.button.text:SetFont("Fonts\\ARIALN.TTF", 48, "OUTLINE, MONOCHROME")
  f.button.text:SetText("")
  f.button.text_points = f.button:CreateFontString(nil, "OVERLAY")
  f.button.text_points:SetPoint("TOPRIGHT", f.button, "TOPRIGHT", -2, -2)
  f.button.text_points:SetFont("Fonts\\ARIALN.TTF", 16, "OUTLINE, MONOCHROME")
  f.button.x = x
  f.button.y = y

  f.button:SetScript("OnClick", function(self)
		ProcessButtonClick(self, x, y)
  end)
  table.insert(buttons, f)
 end

 local f = CreateFrame("FRAME")
 f:SetParent(parent)
 f:SetWidth(framewidth)
 f:SetHeight(frameheight)
 f:SetPoint("CENTER")
 local x,y
 f.buttons = {}
 for x = 0,rows-1 do
  for y = 0,lines-1 do
   createbutton(f,x,y)
  end
 end
 
 return f
end
--

-- main constructor
local function GenerateMainFrame()
 local f
 f = addonTable.CreateDialogFrame(UIParent,true)
 --f.tex = f:CreateTexture()
 --f.tex:SetAllPoints()
 --f.tex:SetTexture("interface\\glues\\loadingscreens\\LoadScreenDeepRunTram")
 f:SetWidth(framewidth + 50)
 f:SetHeight(frameheight + 100)
 f.globaltime = f:CreateFontString()
 f.globaltime:SetFont("Fonts\\MORPHEUS.ttf", 24, "OUTLINE, MONOCHROME")
 f.globaltime:SetPoint("TOPRIGHT", f, "TOPRIGHT", -10, -10)
 f.globaltime:SetWidth(150)
 f.globaltime:SetHeight(40)
 f.globaltime:SetText("0")
 f.currentword = f:CreateFontString()
 f.currentword:SetFont("Fonts\\MORPHEUS.ttf", 24, "OUTLINE, MONOCHROME")
 f.currentword:SetPoint("TOPLEFT", f, "TOPLEFT", 10, -10)
 f.currentword:SetWidth(100)
 f.currentword:SetHeight(40)
 f.currentword:SetText("_")
 f.buttonsframe = GenerateButtonsframe(f)

 local function createsideframe()
  local sf = addonTable.CreateDialogFrame(f)
  sf:SetWidth(framewidth/1.5)
  sf:SetHeight(frameheight/1.5)
  sf.name = sf:CreateFontString()
  sf.name:SetFont("Fonts\\MORPHEUS.ttf", 24, "OUTLINE, MONOCHROME")
  sf.name:SetPoint("TOP",sf,"TOP",0,-15)
  sf.portrait = sf:CreateTexture()
  sf.portrait:SetWidth(50)
  sf.portrait:SetHeight(50)
  sf.portrait:SetPoint("TOP",sf.name,"BOTTOM",0,-10) 
  sf.score = sf:CreateFontString()
  sf.score:SetFont("Fonts\\MORPHEUS.ttf", 48, "OUTLINE, MONOCHROME")
  sf.score:SetPoint("BOTTOM",sf,"BOTTOM",0,15)
  sf:SetBackdropColor(0,0,0,1) 
  return sf
 end
 
 --left side frame
 f.leftframe = createsideframe()
 f.leftframe:SetPoint("RIGHT", f, "LEFT")
 f.leftframe.Refresh = function()
  f.leftframe.name:SetText(UnitName("player"))
   SetPortraitTexture(f.leftframe.portrait, "player")
  f.leftframe.score:SetText("Total\nscore: "..scorecolor..totalscore.."|r")
 end
  
 --right side frame
 f.rightframe = createsideframe()
 f.rightframe:SetPoint("LEFT", f, "RIGHT")
 f.rightframe.Refresh = function()
  if gametype == "singleplayer" then
   f.rightframe:Hide()
  else
   f.rightframe.name:SetText(addonTable.opponent.name)
   f.rightframe.score:SetText("Total\nscore: "..scorecolor..addonTable.opponent.score.."|r")
   f.rightframe:Show()
  end
 end
 --SetPortraitTexture(f.rightframe.portrait, "player") 
 
 f.closebtn = CreateFrame("BUTTON", "Wowzzle_closebtn", f, "OptionsListButtonTemplate")
 f.closebtn:SetParent(f)
 f.closebtn:SetWidth(100)
 f.closebtn:SetHeight(20)
 f.closebtn:SetPoint("BOTTOMLEFT", f, "BOTTOMLEFT", 10, 10)
 f.closebtn:SetText("Quit Game")
 f.closebtn:SetScript("OnClick", function(self)
  gametype = "singleplayer"
  ResetGame()
  addonTable.ResetMultiplayerInterface()
  f:Hide()
  addonTable.Mainmenu_Initialize()
 end)
 f:Show()
 Wowzzle_Mainframe = f
end
--

addonTable.Wowzzle_Initialize = function(arg)
 addonTable:Timer_Init()
 if Wowzzle_Mainframe then 
  Wowzzle_Mainframe:Show()
 else
  GenerateMainFrame()
 end
 if Wowzzle_Menuframe then
  Wowzzle_Menuframe:Hide()
 end
 ResetGame()
 GenerateGrid()
 if arg ~= "multiplayer" then
  gametype = "singleplayer"
  addonTable.StartGame()
 end
 PlaySoundFile("Sound\\Spells\\SimonGame_Visual_LevelStart.wav")
end


addonTable.GetGrid = function()
 if not Wowzzle_Mainframe then return end

 local s = ""
 table.foreach(buttons, function(_,v)
  s = s..v.button.text:GetText()
 end)
 return s
end

addonTable.SetGrid = function(s)
 if not Wowzzle_Mainframe then GenerateMainFrame() end

 local i = 1
 table.foreach(buttons, function(_,v)
  v.button.text:SetText(s:sub(i, i))
  i = i + 1
 end)
end

addonTable.InitGame = function(arg)
 gametype = arg
 addonTable.Wowzzle_Initialize(arg)
 addonTable.RefreshSideFrames()
end

addonTable.StartGame = function()
 addonTable.Timer_Start(TIMER_GLOBAL, game_timelimit, addonTable.EndGame)
 EnableButtons()
end

addonTable.EndGame = function(arg)
 if arg == "pingtimeout" then
  -- connection lost
  PlaySoundFile("Sound\\Doodad\\Goblin_Lottery_Open01.wav")
  addonTable.ShowGameResults(arg, 0, {}, addonTable.opponent)
 else -- normal game over
  addonTable.ShowGameResults(gametype, totalscore, wordsfound, addonTable.opponent)
  if gametype == "multiplayer" then
   SendAddonMessage(C["prefix"], C["endresult"]..totalscore.."@"..table.concat(wordsfound," "), "WHISPER", addonTable.opponent.name)
  end
 end
 DisableButtons()
 addonTable.ResetMultiplayerInterface()
 ResetGame()
 Wowzzle_Mainframe:Hide()
 if Wowzzle_Menuframe then
  Wowzzle_Menuframe:Show()
 end
end